package foo;

import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.jbpm.pvm.internal.cfg.SpringConfiguration;
import org.jbpm.api.*;
import org.jbpm.api.task.Task;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;
import java.util.*;

/**
 * User: Penkov Vladimir
 * Date: 24.12.2009
 * Time: 12:10:46
 */
public class WorkflowService {

    private List<String> processDefinitions;
    private Log logger = LogFactory.getLog(WorkflowService.class);


    private SpringConfiguration springConfiguration;

    private RepositoryService repositoryService;

    private ExecutionService executionService;

    private TaskService taskService;


    @PostConstruct
    public void setupProcessDefinitions() {
        springConfiguration.buildProcessEngine();
        for (String processDefinition : processDefinitions) {
            NewDeployment deployment = repositoryService.createDeployment();
            deployment.addResourceFromClasspath(processDefinition);
            deployment.deploy();
        }
    }

    public ProcessInstance startApprovingProcess(String arg) {
        Map<String, Object> vars = new HashMap<String, Object>();
        vars.put("param", arg);
        return executionService.startProcessInstanceByKey("TestTask", vars);
    }

    public void setProcessDefinitions(List<String> pd) {
        this.processDefinitions = pd;
    }

    /**
     *   ,   .        
     *   jpdl .  ,    :
     * <pre>
     *     &lt;task name="approveTask" assignee="cms-page_publisher">
     * </pre>
     *
     *    ,    "cms-page_publisher".
     * @param roles  
     * @return  
     */
    public List<Task> getAssignedTask(String... roles) {
        List<Task> res = new ArrayList<Task>();
        for (String role : roles) {
            res.addAll(taskService.findPersonalTasks(role));
        }
        return res;
    }


    public Task getTask(String taskId) {
        return taskService.getTask(taskId);
    }

    public Set<String> getTaskOutcomes(String taskId) {
        return taskService.getOutcomes(taskId);
    }

    public void completeTask(String taskId, String outcome) {
        taskService.completeTask(taskId, outcome);
    }

    public Map<String, Object> getTaskVariables(String taskId) {
        return taskService.getVariables(taskId, taskService.getVariableNames(taskId));
    }

    public void setSpringConfiguration(SpringConfiguration springConfiguration) {
        this.springConfiguration = springConfiguration;
    }

    public void setRepositoryService(RepositoryService repositoryService) {
        this.repositoryService = repositoryService;
    }

    public void setExecutionService(ExecutionService executionService) {
        this.executionService = executionService;
    }

    public void setTaskService(TaskService taskService) {
        this.taskService = taskService;
    }
}
